print(
    " ".join(
        ['/usr/local/bin/sphinx-build', '-b', 'dirhtml', '-d',
         '/home/obestwalter/projects/linda/bilderwerkstatt_ravensburg.de/'
         '__build/doctrees',
         '/home/obestwalter/projects/linda/bilderwerkstatt_ravensburg.de',
         '/home/obestwalter/projects/linda/bilderwerkstatt_ravensburg.de/'
         '__build/html']
    ))
